package ru.landgrafhomyak.db.serdha0.api.misc

import kotlin.jvm.JvmName
import ru.landgrafhomyak.db.serdha0.api.queries.SelectQuery
import ru.landgrafhomyak.db.serdha0.api.queries.InsertQuery
import ru.landgrafhomyak.db.serdha0.api.queries.UpdateQuery
import ru.landgrafhomyak.db.serdha0.api.queries.DeleteQuery


/**
 * Utility interface to mark queries (and table) that can be used as the source for other queries.
 *
 * @param UE Type of extension provided by user ([about user extensions](https://git.landgrafhomyak.ru/SERDHA/serdha-0-api.kt/wiki/User-extensions)).
 *
 * @see SelectQuery
 * @see InsertQuery.FromSubquery.Constructor.selector
 * @see UpdateQuery.FromSubquery.Constructor.selector
 * @see DeleteQuery.FromSubquery.Constructor.selector
 */
@Suppress("ClassName")
public interface _Selectable<out UE : Any> {
	/**
	 * Getter of an extension provided by user ([about user extensions](https://git.landgrafhomyak.ru/SERDHA/serdha-0-api.kt/wiki/User-extensions)).
	 */
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("uExt")
	public val uExt: UE
}