package ru.landgrafhomyak.db.serdha0.api.queries

import kotlin.jvm.JvmName
import ru.landgrafhomyak.db.serdha0.api.misc.Column
import ru.landgrafhomyak.db.serdha0.api.misc.DatabaseType
import ru.landgrafhomyak.db.serdha0.api.misc.QueryParam
import ru.landgrafhomyak.db.serdha0.api.misc._Selectable
import ru.landgrafhomyak.db.serdha0.api.runtime.InputRow
import ru.landgrafhomyak.db.serdha0.api.table.RowId
import ru.landgrafhomyak.db.serdha0.api.table.Table

public interface DeleteQuery<out qUE : Any> : _Query<qUE> {
	public interface SingleRow<out qUE : Any> : DeleteQuery<qUE>, _Query.Params2Table<qUE> {
		public interface Constructor<qUE : Any, ttUE : Any> : _Query.Constructor<SingleRow<qUE>> {
			override fun createQuery(internalQueries: _Query.Constructor.Scope, outQueries: _Query.Constructor.Scope): SingleRow<qUE> =
				outQueries.deleteSingleRow(this)

			@Suppress("INAPPLICABLE_JVM_NAME")
			@get:JvmName("targetTable")
			public val targetTable: Table<ttUE, *>

			public fun createDeleteSingleRowWithReturning(context: Scope<qUE, ttUE>): qUE

			public interface Scope<qUE : Any, ttUE : Any> {
				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("tt")
				public val tt: Table<ttUE, *>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("types")
				public val types: DatabaseType.Provider
				public fun <RT, DT : DatabaseType<RT>> queryParam(name: String, type: DT): QueryParam<RT, DT, qUE>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("rowToRemove")
				public val rowToRemove: QueryParam<RowId<ttUE>, DatabaseType.ROW_ID<ttUE>, qUE>

				public fun <RT, DT : DatabaseType<RT>> returnRemovedValue(name: String, column: Column<RT, DT, ttUE>): Column<RT, DT, qUE>
				public fun <RT, DT : DatabaseType<RT>> returnParam(name: String, column: QueryParam<RT, DT, ttUE>): Column<RT, DT, qUE>

				public fun ifNoRowSkip()
				public fun ifNoRowJustReturn()
				public fun ifNoRowThrow()
			}
		}
	}

	public interface FromSubquery<out qUE : Any> : DeleteQuery<qUE>, _Query.Params2Table<qUE> {
		public interface Constructor<qUE : Any, ttUE : Any, sUE : Any> : _Query.Constructor<FromSubquery<qUE>> {
			override fun createQuery(internalQueries: _Query.Constructor.Scope, outQueries: _Query.Constructor.Scope): FromSubquery<qUE> =
				outQueries.deleteFromSubquery(this)

			@Suppress("INAPPLICABLE_JVM_NAME")
			@get:JvmName("rowToRemove")
			public val targetTable: Table<ttUE, *>

			@Suppress("INAPPLICABLE_JVM_NAME")
			@get:JvmName("selector")
			public val selector: _Selectable<sUE>

			public fun createDeleteFromSubqueryWithReturning(context: Scope<qUE, ttUE, sUE>): qUE

			public interface Scope<qUE : Any, ttUE : Any, sUE : Any> {
				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("tt")
				public val tt: Table<ttUE, *>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("src")
				public val src: _Selectable<sUE>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("types")
				public val types: DatabaseType.Provider
				public fun <RT, DT : DatabaseType<RT>> queryParam(name: String, type: DT): QueryParam<RT, DT, qUE>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("selectorParams")
				public val selectorParams: InputRow.WithRedirect<sUE, qUE>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("_getSelectorColumn")
				@set:JvmName("selectorColumn")
				public var selectorColumn: Column<RowId<ttUE>, DatabaseType.ROW_ID<ttUE>, sUE>

				public fun <RT, DT : DatabaseType<RT>> returnFromSubquery(name: String, column: Column<RT, DT, sUE>): Column<RT, DT, qUE>
				public fun <RT, DT : DatabaseType<RT>> returnRemovedValue(name: String, column: Column<RT, DT, ttUE>): Column<RT, DT, qUE>
				public fun <RT, DT : DatabaseType<RT>> returnParam(name: String, column: QueryParam<RT, DT, ttUE>): Column<RT, DT, qUE>

				public fun ifNoRowSkip()
				public fun ifNoRowJustReturn()
				public fun ifNoRowThrow()
			}
		}
	}

	public interface MultipleRows<out qUE : Any> : DeleteQuery<qUE>, _Query.Table2Void<qUE> {
		public interface Constructor<qUE : Any, ttUE : Any> : _Query.Constructor<MultipleRows<qUE>> {
			override fun createQuery(internalQueries: _Query.Constructor.Scope, outQueries: _Query.Constructor.Scope): MultipleRows<qUE> =
				outQueries.deleteMultipleRows(this)

			@Suppress("INAPPLICABLE_JVM_NAME")
			@get:JvmName("targetTable")
			public val targetTable: Table<ttUE, *>

			public fun createMultipleRows(context: Scope<qUE, ttUE>): qUE

			public interface Scope<qUE : Any, ttUE : Any> {
				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("tt")
				public val tt: Table<ttUE, *>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("rowsToRemove")
				public val rowsToRemove: QueryParam<RowId<ttUE>, DatabaseType.ROW_ID<ttUE>, qUE>

				public fun ifNoRowSkip()
				public fun ifNoRowThrow()
			}
		}
	}
}